/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_serial.c
* Version      : CodeGenerator for RL78/G14 V2.05.03.02 [06 Nov 2018]
* Device(s)    : R5F104ML
* Tool-Chain   : GCCRL78
* Description  : This file implements device driver for Serial module.
* Creation Date: 
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_serial.h"
/* Start user code for include. Do not edit comment generated here */
#include "r_cg_dtc.h"
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
volatile uint8_t * gp_uart3_tx_address;        /* uart3 transmit buffer address */
volatile uint16_t  g_uart3_tx_count;           /* uart3 transmit data number */
volatile uint8_t * gp_uart3_rx_address;        /* uart3 receive buffer address */
volatile uint16_t  g_uart3_rx_count;           /* uart3 receive data number */
volatile uint16_t  g_uart3_rx_length;          /* uart3 receive data length */
/* Start user code for global. Do not edit comment generated here */
SemaphoreHandle_t  g_uart3_tx_semaphore;       /* uart3 send semaphore handle */
SemaphoreHandle_t  g_uart3_rx_semaphore;       /* uart3 receive semaphore handle */
volatile uint8_t * gp_uart3_rx_usr_error_type; /* uart3 receive error flags address (user side) */

void U_UART3_DTC_Send_Stop(void);              /* for internal use */
void U_UART3_DTC_Receive_Stop(void);           /* for internal use */

#define R_UART3_Receive R_UART3_Receive_DO_NOT_USE
#define R_UART3_Send R_UART3_Send_DO_NOT_USE
MD_STATUS R_UART3_Receive(uint8_t * const rx_buf, uint16_t rx_num);
MD_STATUS R_UART3_Send(uint8_t * const tx_buf, uint16_t tx_num);
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: R_SAU1_Create
* Description  : This function initializes the SAU1 module.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_SAU1_Create(void)
{
    SAU1EN = 1U;    /* supply SAU1 clock */
    NOP();
    NOP();
    NOP();
    NOP();
    SPS1 = _0000_SAU_CK00_FCLK_0 | _0000_SAU_CK01_FCLK_0;
    R_UART3_Create();
}

/***********************************************************************************************************************
* Function Name: R_UART3_Create
* Description  : This function initializes the UART3 module.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_UART3_Create(void)
{
    ST1 |= _0008_SAU_CH3_STOP_TRG_ON | _0004_SAU_CH2_STOP_TRG_ON;    /* disable UART3 receive and transmit */
    STMK3 = 1U;    /* disable INTST3 interrupt */
    STIF3 = 0U;    /* clear INTST3 interrupt flag */
    SRMK3 = 1U;    /* disable INTSR3 interrupt */
    SRIF3 = 0U;    /* clear INTSR3 interrupt flag */
    SREMK3 = 1U;   /* disable INTSRE3 interrupt */
    SREIF3 = 0U;   /* clear INTSRE3 interrupt flag */
    /* Set INTST3 low priority */
    STPR13 = 1U;
    STPR03 = 1U;
    /* Set INTSR3 low priority */
    SRPR13 = 1U;
    SRPR03 = 1U;
    /* Set INTSRE3 low priority */
    SREPR13 = 1U;
    SREPR03 = 1U;
    SMR12 = _0020_SAU_SMRMN_INITIALVALUE | _0000_SAU_CLOCK_SELECT_CK00 | _0000_SAU_TRIGGER_SOFTWARE |
            _0002_SAU_MODE_UART | _0001_SAU_BUFFER_EMPTY;
    SCR12 = _8000_SAU_TRANSMISSION | _0000_SAU_INTSRE_MASK | _0300_SAU_PARITY_ODD | _0080_SAU_LSB | _0010_SAU_STOP_1 |
            _0007_SAU_LENGTH_8;
    SDR12 = _0E00_UART3_TRANSMIT_DIVISOR;
    NFEN0 |= _40_SAU_RXD3_FILTER_ON;
    SIR13 = _0004_SAU_SIRMN_FECTMN | _0002_SAU_SIRMN_PECTMN | _0001_SAU_SIRMN_OVCTMN;    /* clear error flag */
    SMR13 = _0020_SAU_SMRMN_INITIALVALUE | _0000_SAU_CLOCK_SELECT_CK00 | _0100_SAU_TRIGGER_RXD | _0000_SAU_EDGE_FALL |
            _0002_SAU_MODE_UART | _0000_SAU_TRANSFER_END;
    SCR13 = _4000_SAU_RECEPTION | _0400_SAU_INTSRE_ENABLE | _0300_SAU_PARITY_ODD | _0080_SAU_LSB | _0010_SAU_STOP_1 |
            _0007_SAU_LENGTH_8;
    SDR13 = _0E00_UART3_RECEIVE_DIVISOR;
    SO1 |= _0004_SAU_CH2_DATA_OUTPUT_1;
    SOL1 |= _0000_SAU_CHANNEL2_NORMAL;    /* output level normal */
    SOE1 |= _0004_SAU_CH2_OUTPUT_ENABLE;    /* enable UART3 output */
    /* Set RxD3 pin */
    PM14 |= 0x08U;
    /* Set TxD3 pin */
    P14 |= 0x10U;
    PM14 &= 0xEFU;
}

/***********************************************************************************************************************
* Function Name: R_UART3_Start
* Description  : This function starts the UART3 module operation.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_UART3_Start(void)
{
    SO1 |= _0004_SAU_CH2_DATA_OUTPUT_1;    /* output level normal */
    SOE1 |= _0004_SAU_CH2_OUTPUT_ENABLE;    /* enable UART3 output */
    SS1 |= _0008_SAU_CH3_START_TRG_ON | _0004_SAU_CH2_START_TRG_ON;    /* enable UART3 receive and transmit */
    STIF3 = 0U;    /* clear INTST3 interrupt flag */
    SRIF3 = 0U;    /* clear INTSR3 interrupt flag */
    SREIF3 = 0U;   /* clear INTSRE3 interrupt flag */
    STMK3 = 0U;    /* enable INTST3 interrupt */
    SRMK3 = 0U;    /* enable INTSR3 interrupt */
    SREMK3 = 0U;   /* enable INTSRE3 interrupt */
}

/***********************************************************************************************************************
* Function Name: R_UART3_Stop
* Description  : This function stops the UART3 module operation.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_UART3_Stop(void)
{
    STMK3 = 1U;    /* disable INTST3 interrupt */
    SRMK3 = 1U;    /* disable INTSR3 interrupt */
    SREMK3 = 1U;   /* disable INTSRE3 interrupt */
    ST1 |= _0008_SAU_CH3_STOP_TRG_ON | _0004_SAU_CH2_STOP_TRG_ON;    /* disable UART3 receive and transmit */
    SOE1 &= ~_0004_SAU_CH2_OUTPUT_ENABLE;    /* disable UART3 output */
    STIF3 = 0U;    /* clear INTST3 interrupt flag */
    SRIF3 = 0U;    /* clear INTSR3 interrupt flag */
    SREIF3 = 0U;   /* clear INTSRE3 interrupt flag */
   
}

/***********************************************************************************************************************
* Function Name: R_UART3_Receive
* Description  : This function receives UART3 data.
* Arguments    : rx_buf -
*                    receive buffer pointer
*                rx_num -
*                    buffer size
* Return Value : status -
*                    MD_OK or MD_ARGERROR
***********************************************************************************************************************/
MD_STATUS R_UART3_Receive(uint8_t * const rx_buf, uint16_t rx_num)
{
    MD_STATUS status = MD_OK;

    if (rx_num < 1U)
    {
        status = MD_ARGERROR;
    }
    else
    {
        g_uart3_rx_count = 0U;
        g_uart3_rx_length = rx_num;
        gp_uart3_rx_address = rx_buf;
    }

    return (status);
}

/***********************************************************************************************************************
* Function Name: R_UART3_Send
* Description  : This function sends UART3 data.
* Arguments    : tx_buf -
*                    transfer buffer pointer
*                tx_num -
*                    buffer size
* Return Value : status -
*                    MD_OK or MD_ARGERROR
***********************************************************************************************************************/
MD_STATUS R_UART3_Send(uint8_t * const tx_buf, uint16_t tx_num)
{
    MD_STATUS status = MD_OK;

    if (tx_num < 1U)
    {
        status = MD_ARGERROR;
    }
    else
    {
        gp_uart3_tx_address = tx_buf;
        g_uart3_tx_count = tx_num;
        SMR12 |= _0001_SAU_BUFFER_EMPTY;
        STMK3 = 1U;    /* disable INTST3 interrupt */
        TXD3 = *gp_uart3_tx_address;
        gp_uart3_tx_address++;
        g_uart3_tx_count--;
        STMK3 = 0U;    /* enable INTST3 interrupt */
    }

    return (status);
}

/* Start user code for adding. Do not edit comment generated here */
/******************************************************************************
* Function Name: U_UART3_ASYNC_Start
* Description  : This function starts the UART3 module with ASYNC operation.
* Arguments    : None
* Return Value : None
******************************************************************************/
void U_UART3_ASYNC_Start(SemaphoreHandle_t tx_sem, SemaphoreHandle_t rx_sem)
{
    g_uart3_tx_semaphore = tx_sem;
    g_uart3_rx_semaphore = rx_sem;
    R_UART3_Start();
    /*STMK3 = 1U;*//* disable INTST3 interrupt *//* unnecessary */
    SRMK3 = 1U;    /* disable INTSR3 interrupt */
    SREMK3 = 1U;   /* disable INTSRE3 interrupt */
}

/******************************************************************************
* Function Name: U_UART3_DTC_Send
* Description  : This function sends UART3 data using DTCD1.
* Arguments    : tx_buf -
*                    transfer buffer pointer
*                tx_num -
*                    buffer size
* Return Value : status -
*                    MD_OK or MD_ARGERROR
******************************************************************************/
MD_STATUS U_UART3_DTC_Send(uint8_t * tx_buf, uint16_t tx_num)
{
    MD_STATUS status = MD_OK;

    if (tx_num < 1U && 256U < tx_num)
    {
        status = MD_ARGERROR;
    }
    else
    {
        if (1 < tx_num)
        {
            U_DTCD1_UserInit( tx_buf + 1, tx_num - 1 );
            R_DTCD1_Start();
            SMR12 |= _0001_SAU_BUFFER_EMPTY;
        }
        else
        {
            SMR12 &= ~_0001_SAU_BUFFER_EMPTY;
        }
        STIF3 = 0U;    /* clear INTST3 interrupt flag */
        STMK3 = 0U;    /* enable INTST3 interrupt */
        TXD3 = *tx_buf;    /* also raise INTST3 interrupt */
    }
    return status;
}

/******************************************************************************
* Function Name: U_UART3_DTC_Send_Abort
* Description  : This function aborts the UART3 data transmission
* Arguments    : None
* Return Value : None
* Note         : Please call this after send timeout (but it may not occur) if no retry.
******************************************************************************/
void U_UART3_DTC_Send_Abort(void)
{
    U_UART3_DTC_Send_Stop();
    R_DTCD1_Stop();    /* forcibly stop DTCD1 */
}

/******************************************************************************
* Function Name: U_UART3_DTC_Send_Stop
* Description  : This function stops the UART3 data transmission
* Arguments    : None
* Return Value : None
* Note         : This is called from others internally.
******************************************************************************/
void U_UART3_DTC_Send_Stop(void)
{
    STMK3 = 1U;    /* disable INTST3 interrupt */
}

/******************************************************************************
* Function Name: U_UART3_DTC_Receive
* Description  : This function receives UART3 data using DTCD0.
* Arguments    : rx_buf -
*                    receive buffer pointer
*                rx_num -
*                    buffer size
*                p_err_type -
*                    error type variable pointer
* Return Value : status -
*                    MD_OK or MD_ARGERROR
******************************************************************************/
MD_STATUS U_UART3_DTC_Receive(volatile uint8_t * rx_buf, uint16_t rx_num, volatile uint8_t * p_err_type)
{
    MD_STATUS status = MD_OK;

    if (rx_num < 1U && 256U < rx_num)
    {
        status = MD_ARGERROR;
    }
    else
    {
        gp_uart3_rx_usr_error_type = p_err_type;
        (void) RXD3;        /* dummy read */
        SIR13 = 0x0007U;    /* clear UART3 receive error flags */
        SRIF3 = 0U;    /* clear INTSR3 interrupt flag */
        SREIF3 = 0U;   /* clear INTSRE3 interrupt flag */
        U_DTCD0_UserInit( rx_buf, rx_num );
        R_DTCD0_Start();
        SRMK3 = 0U;    /* enable INTSR3 interrupt */
        SREMK3 = 0U;   /* enable INTSRE3 interrupt */
    }
    return status;
}

/******************************************************************************
* Function Name: U_UART3_DTC_Receive_Abort
* Description  : This function aborts the UART3 data reception
* Arguments    : None
* Return Value : None
* Note         : Please call this after receive timeout if no retry.
******************************************************************************/
void U_UART3_DTC_Receive_Abort(void)
{
    U_UART3_DTC_Receive_Stop();
    R_DTCD0_Stop();    /* forcibly stop DTCD0 */
}

/******************************************************************************
* Function Name: U_UART3_DTC_Receive_Stop
* Description  : This function stops the UART3 data reception
* Arguments    : None
* Return Value : None
* Note         : This is called from others internally.
******************************************************************************/
void U_UART3_DTC_Receive_Stop(void)
{
    SRMK3 = 1U;    /* disable INTSR3 interrupt */
    SREMK3 = 1U;   /* disable INTSRE3 interrupt */
}
/* End user code. Do not edit comment generated here */
